/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: DelegatorZipStreamAccessor.java 4483 2009-01-26 08:47:13Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.stream.impl.delegator.zip;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.ow2.util.stream.api.IStreamAccessor;
import org.ow2.util.stream.impl.delegator.DelegatorStreamAccessor;

/**
 * @author Gael Lalire
 */
public class DelegatorZipStreamAccessor extends DelegatorStreamAccessor implements IStreamAccessor {

    private static final int BUFFER_SIZE = 8192;

    private String fileName;

    private static InputStream getInputStream(final IStreamAccessor parentFileAccessor, final String fileName)
            throws IOException {
        InputStream parentInputStream;
        try {
            parentInputStream = parentFileAccessor.getInputStream();
        } catch (FileNotFoundException e) {
            return null;
        }
        ZipInputStream zipInputStream = new ZipInputStream(parentInputStream);
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        if (zipEntry == null) {
            return null;
        }
        while (!zipEntry.getName().equals(fileName)) {
            zipEntry = zipInputStream.getNextEntry();
            if (zipEntry == null) {
                return null; // a null input stream mean that file does not
                             // exists
            }
        }
        return zipInputStream;
    }

    public DelegatorZipStreamAccessor(final IStreamAccessor parentFileAccessor, final String fileName) throws ZipException,
            IOException {
        super(parentFileAccessor, getInputStream(parentFileAccessor, fileName)); // the file will be extracted in a tmp file
        this.fileName = fileName;
    }

    @Override
    protected void writeToParent(final IStreamAccessor parentFileAccessor, final InputStream localFileInputStream)
            throws IOException {
        ZipOutputStream zipOutputStream = new ZipOutputStream(parentFileAccessor.getTemporaryOutputStream());
        InputStream parentInputStream = null;
        try {
            parentInputStream = parentFileAccessor.getInputStream();
        } catch (FileNotFoundException e) {
        }
        byte[] buf = new byte[BUFFER_SIZE];
        boolean fileFound = false;

        if (parentInputStream != null) {
            ZipInputStream zipInputStream = new ZipInputStream(parentInputStream);
            ZipEntry zipEntry = zipInputStream.getNextEntry();

            while (zipEntry != null) {
                InputStream inputStream;
                boolean replaceZipFile = zipEntry.getName().equals(fileName);
                if (!replaceZipFile) {
                    // keep other file in zip
                    inputStream = zipInputStream;
                } else {
                    // replace file in zip
                    zipEntry = new ZipEntry(fileName);
                    inputStream = localFileInputStream;
                    fileFound = true;
                }
                zipOutputStream.putNextEntry(zipEntry);
                int read = inputStream.read(buf);
                while (read != -1) {
                    zipOutputStream.write(buf, 0, read);
                    read = inputStream.read(buf);
                }
                if (replaceZipFile) {
                    inputStream.close();
                } else {
                    zipInputStream.closeEntry();
                }
                zipOutputStream.closeEntry();
                zipEntry = zipInputStream.getNextEntry();
            }
            zipInputStream.close();
        }
        // add file in zip
        if (!fileFound) {
            ZipEntry newZipEntry = new ZipEntry(fileName);
            zipOutputStream.putNextEntry(newZipEntry);

            int read = localFileInputStream.read(buf);
            while (read != -1) {
                zipOutputStream.write(buf, 0, read);
                read = localFileInputStream.read(buf);
            }
            localFileInputStream.close();
            zipOutputStream.closeEntry();
        }

        // close input stream before
        zipOutputStream.close();
    }

}
